/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.container;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Scanner;

import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;

/**
 * Checks text entered into a StyledText widget for line breaks, sends
 * split text to abstract method handleSplitText().
 *  <p>
 * VerifyListener interface methods break up the text into lines. Each line is sent individually
 * to the handleSplitText() method when the KeyListener interface method keyReleased() is called.
 * This is because if the StyledText widget is disposed before the key is released, an SWT
 * exception occurs.
 * <p>
 * The listener should be set on the StyledText widget as follows:
 * <code><pre>
 * 		StyledText styledText = new SourceViewer().getTextWidget();
 *		LineBreakListener lineBreakListener = new LineBreakListener() {
 *			public void handleSplitText(String text) {
 *				... some operation ...
 *			}
 *		};
 *
 *		styledText.addVerifyListener(lineBreakListener);
 *		styledText.addKeyListener(lineBreakListener);
 * </pre></code>
 *
 * @author p.ciardelli
 * @created 19.05.2008
 * @version 1.0
 */
public abstract class LineBreakListener implements VerifyListener, KeyListener {

	private static String LINE_BREAK = System.getProperty("line.separator");
	
	
	/**
	 * {@inheritDoc}
	 *
	 * Checks for 3 conditions of name viewer text:
	 *   1) no line breaks in text
	 *   2) line break at the end of the text
	 *   3) line break in the middle of the text
	 * @see org.eclipse.swt.events.VerifyListener#verifyText(org.eclipse.swt.events.VerifyEvent)
	 */
	public void verifyText(VerifyEvent e) {
						
		String verifyText = e.text;
		
		StyledText styledText = (StyledText) e.widget;
		String widgetText = styledText.getText(); 
		int cursorPosition = styledText.getCaretOffset();
		
		// If no line break in verifyText, Parse text normally
		if (!verifyText.contains(LINE_BREAK)) {
			return;
		}
				
		// Don't add verifyText without further processing
		e.doit = false;
		
		// If user has entered return at the end of the line,
		//  make a new, empty synonym
		if (widgetText.length() == cursorPosition &&
				verifyText.equals(LINE_BREAK)) {
			handleSplitText("");
			return;
		}
		
		// Concatenate old and new texts
		String textForSplitting = widgetText.substring(0, cursorPosition) 
				+ verifyText
				+ widgetText.substring(cursorPosition);
		
		// Split on line breaks
		Scanner scanner = new Scanner( textForSplitting );
		scanner.useDelimiter(LINE_BREAK);
		
		// Put first string into name viewer
		styledText.setText(scanner.next());	
		
		// Start new synonyms with the rest
		while (scanner.hasNext()) {
			handleSplitText(scanner.next());
		}
	}
	
	/**
	 * <p>handleSplitText</p>
	 *
	 * @param text a {@link java.lang.String} object.
	 */
	abstract public void handleSplitText(String text);
}
