package eu.etaxonomy.taxeditor.editor.view.derivate.contextMenu;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;

import eu.etaxonomy.cdm.api.service.molecular.ISequenceService;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * Context menu for the SingleReads in the derivate hierarchy.
 *
 */
public class SingleReadSequenceContextMenu extends CompoundContributionItem {

    private enum CommandType{
        COPY_TO_CLIPBOARD,
        REMOVE_FROM_SEQUENCE,
        ADD_TO_SEQUENCE
    }

    @Override
    protected IContributionItem[] getContributionItems() {
        IContributionItem[] contributionItems = new IContributionItem[] {
                new ContributionItem() {
                    @Override
                    public void fill(Menu menu, int index) {
                        final IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
                        final IEditorPart activeEditor = window.getActivePage().getActiveEditor();
                        final ISelection selection = window.getActivePage().getSelection();
                        TreeNode selectedTreeNode = EditorUtil.getTreeNodeOfSelection(selection);
                        TreeNode clipboardNode = EditorUtil.getTreeNodeOfSelection(LocalSelectionTransfer.getTransfer().getSelection());
                        if(selectedTreeNode!=null){
                            //context menu for Sequence
                            if(selectedTreeNode.getValue() instanceof Sequence
                                    && clipboardNode!=null && clipboardNode.getValue() instanceof SingleRead){
                                MenuItem item = new MenuItem(menu, SWT.NONE);
                                item.setText(Messages.SingleReadSequenceContextMenu_REUSE_SINGLE_READ_HERE);
                                item.addSelectionListener(new WidgetSelectionListener(CommandType.ADD_TO_SEQUENCE));
                            }
                            else if(selectedTreeNode.getValue() instanceof SingleRead){
                                MenuItem item = new MenuItem(menu, SWT.NONE);
                                item.setText(Messages.SingleReadSequenceContextMenu_REUSE_FOR_SEQUENCE);
                                item.addSelectionListener(new WidgetSelectionListener(CommandType.COPY_TO_CLIPBOARD));
                                if(activeEditor instanceof DerivateView
                                        && ((DerivateView) activeEditor).getMultiLinkSingleReads().contains(selectedTreeNode.getValue())){
                                    MenuItem unlinkItem = new MenuItem(menu, SWT.NONE);
                                    unlinkItem.setText(Messages.SingleReadSequenceContextMenu_REMOVE_FROM_SEQUENCE);
                                    unlinkItem.addSelectionListener(new WidgetSelectionListener(CommandType.REMOVE_FROM_SEQUENCE));
                                }
                            }
                        }
                    }
                }
        };
        return contributionItems;
    }

    private class WidgetSelectionListener extends SelectionAdapter{
        private final CommandType commandType;

        /**
         * @param selectedTreeNode
         * @param clipboardNode
         * @param commandType
         */
        public WidgetSelectionListener(CommandType commandType) {
            this.commandType = commandType;
        }

        @Override
        public void widgetSelected(SelectionEvent e) {
            IEditorPart activeEditor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
            ISelection clipBoardSelection = LocalSelectionTransfer.getTransfer().getSelection();
            final ISelection selection = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getSelection();
            TreeNode clipBoardTreeNode = EditorUtil.getTreeNodeOfSelection(clipBoardSelection);
            TreeNode selectedTreeNode = EditorUtil.getTreeNodeOfSelection(selection);
            switch (commandType) {
            case REMOVE_FROM_SEQUENCE:
                if(activeEditor.isDirty()){
                    MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
                    return;
                }
                if(selectedTreeNode!=null && selectedTreeNode.getParent()!=null &&
                        selectedTreeNode.getValue() instanceof SingleRead && selectedTreeNode.getParent().getValue() instanceof Sequence) {
                    Sequence sequence = (Sequence) selectedTreeNode.getParent().getValue();
                    sequence.removeSingleRead((SingleRead) selectedTreeNode.getValue());
                    CdmStore.getService(ISequenceService.class).saveOrUpdate(sequence);
                    if(activeEditor instanceof DerivateView) {
                        DerivateView derivateView = (DerivateView)activeEditor;
                        derivateView.getConversationHolder().commit();
                        derivateView.refreshTree();
                    }
                }

                break;
            case ADD_TO_SEQUENCE:
                if(activeEditor.isDirty()){
                    MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
                    return;
                }
                if(clipBoardTreeNode!=null && clipBoardTreeNode.getValue() instanceof SingleRead
                        && selectedTreeNode!=null && selectedTreeNode.getValue() instanceof Sequence){
                    SingleRead singleRead = (SingleRead)clipBoardTreeNode.getValue();
                    Sequence sequence = (Sequence)selectedTreeNode.getValue();
                    sequence.addSingleRead(singleRead);
                    CdmStore.getService(ISequenceService.class).saveOrUpdate(sequence);
                    if(activeEditor instanceof DerivateView) {
                        DerivateView derivateView = (DerivateView)activeEditor;
                        derivateView.getConversationHolder().commit();
                        derivateView.refreshTree();
                        LocalSelectionTransfer.getTransfer().setSelection(null);
                    }
                }
                break;
            case COPY_TO_CLIPBOARD:
                LocalSelectionTransfer.getTransfer().setSelection(selection);
                break;
            }
        }
    }
}
