/**
 *
 */
package eu.etaxonomy.taxeditor.editor.view.descriptive;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.cdm.model.description.IDescribable;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainerTree;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.TermStore;
import eu.etaxonomy.taxeditor.store.UsageTermCollection;

/**
 * <p>DescriptiveContentProvider class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @version $Id: $
 */
public class DescriptiveContentProvider implements ITreeContentProvider {

	protected static final Object[] NO_CHILDREN = new Object[0];
	protected Map<DescriptionBase<?>, FeatureNodeContainerTree> featureNodeContainerCache;
	private boolean showOnlyIndividualAssociations;

	public DescriptiveContentProvider(Map<DescriptionBase<?>, FeatureNodeContainerTree> featureNodeContainerCache) {
	    this(featureNodeContainerCache, false);
	}
	/**
	 * <p>Constructor for DescriptiveContentProvider.</p>
	 *
	 * @param featureNodeContainerCache a {@link java.util.Map} object.
	 */
	public DescriptiveContentProvider(Map<DescriptionBase<?>, FeatureNodeContainerTree> featureNodeContainerCache, boolean showOnlyIndividualAssociations) {
		this.featureNodeContainerCache = featureNodeContainerCache;
		this.showOnlyIndividualAssociations = showOnlyIndividualAssociations;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
    public Object[] getChildren(Object parentElement) {
		if (parentElement instanceof IDescribable<?>) {
			return getDescriptions((IDescribable<?>) parentElement).toArray();
		}
		else if (parentElement instanceof DescriptionBase<?>) {
			if ( ! ((DescriptionBase<?>) parentElement).isImageGallery()) {
			    DescriptionBase<?> description = (DescriptionBase<?>) parentElement;

				FeatureNodeContainerTree containerTree = getContainerTreeForDesription(description);
				List<FeatureNodeContainer> children = containerTree.getRoot().getChildren();
				//filter out containers with no children
				List<FeatureNodeContainer> childrenWithChildren = new ArrayList<FeatureNodeContainer>();
				for (FeatureNodeContainer featureNodeContainer : children) {
				    if(getChildren(featureNodeContainer).length>0){
				        childrenWithChildren.add(featureNodeContainer);
				    }
                }
				return childrenWithChildren.toArray();
			}
		}
		else if (parentElement instanceof FeatureNodeContainer){
			FeatureNodeContainer container = (FeatureNodeContainer) parentElement;
			if(container.isLeaf()){
				List<DescriptionElementBase> descriptionElements = container.getDescriptionElements();
				List<IndividualsAssociation> individualAssociations = new ArrayList<IndividualsAssociation>();
				for (DescriptionElementBase descriptionElement : descriptionElements) {
				    if(descriptionElement instanceof IndividualsAssociation){
				        individualAssociations.add((IndividualsAssociation) descriptionElement);
				    }
				}
				//sort individual associations by title cache of associated specimens
				Collections.sort(individualAssociations, new Comparator<IndividualsAssociation>() {

                    @Override
                    public int compare(IndividualsAssociation o1, IndividualsAssociation o2) {
                        if(o1==null || o1.getAssociatedSpecimenOrObservation()==null){
                            return -1;
                        }
                        if(o2==null || o2.getAssociatedSpecimenOrObservation()==null){
                            return 1;
                        }
                        return o1.getAssociatedSpecimenOrObservation().compareTo(o2.getAssociatedSpecimenOrObservation());
                    }
                });
				if(showOnlyIndividualAssociations){
				    descriptionElements = new ArrayList<DescriptionElementBase>(individualAssociations);
				}
                return descriptionElements.toArray();
			}else{
				return container.getChildren().toArray();
			}
		}

		return NO_CHILDREN;
	}

	private FeatureNodeContainerTree getContainerTreeForDesription(DescriptionBase<?> description){
		if(! featureNodeContainerCache.containsKey(description)){
			FeatureNodeContainerTree containerTree = new FeatureNodeContainerTree(description, getFeatureTree(description));
			featureNodeContainerCache.put(description, containerTree);
		}
		return featureNodeContainerCache.get(description);
	}

	/** {@inheritDoc} */
	@Override
	public boolean hasChildren(Object element) {
		if (element instanceof DescriptionBase<?>){
		    DescriptionBase<?> description = (DescriptionBase<?>) element;
			FeatureNodeContainerTree containerTree = featureNodeContainerCache.get(description);
			if(containerTree != null && containerTree.getRoot() != null){
				return containerTree.getRoot().getChildren().size() != 0;
			}
		}
		return getChildren(element).length != 0;
	}

	/**
	 * Retrieves the feature tree associated with the given description
	 *
	 * TODO as of now this is always the same thing because feature trees may not be associated
	 * to descriptions yet.
	 *
	 * @param description
	 * @return
	 */
	private FeatureTree getFeatureTree(DescriptionBase description){
		FeatureTree featureTree = null;

		// TODO change this to the feature tree associated with this taxon description
		if (description.hasStructuredData()){
			featureTree = PreferencesUtil.getDefaultFeatureTreeForStructuredDescription();
		}else{
			featureTree = PreferencesUtil.getDefaultFeatureTreeForTextualDescription();
		}

		// create a transient tree with all features if none was selected
		if(featureTree == null){
			featureTree = FeatureTree.NewInstance(TermStore.getTerms(Feature.class));
		}

		return featureTree;
	}

    /**
     * Get all descriptions associated with the given object
     * @param parentElement
     * @return
     */
    protected List<DescriptionBase<?>> getDescriptions(IDescribable<?> parentElement) {
        Set<? extends DescriptionBase<?>> elementDescriptions = parentElement.getDescriptions();
        List<DescriptionBase<?>> resultDescriptions = new ArrayList<DescriptionBase<?>>();
        for(DescriptionBase<?> description : elementDescriptions){
			if(! description.isImageGallery()){
				Set<Marker> descriptionMarkers = description.getMarkers();
				if(descriptionMarkers != null && !descriptionMarkers.isEmpty()) {
					for (Marker marker: descriptionMarkers) {
						if(marker.getMarkerType() != null && !(marker.getMarkerType().getUuid().equals(UsageTermCollection.uuidUseMarkerType))) {
							resultDescriptions.add(description);
						}
					}
				}
				else {
					resultDescriptions.add(description);
				}
			}
		}
        return resultDescriptions;
    }

	/** {@inheritDoc} */
	@Override
    public Object getParent(Object element) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
    public Object[] getElements(Object inputElement) {
		return getChildren(inputElement);
	}

	/**
	 * <p>dispose</p>
	 */
	@Override
    public void dispose() {
		featureNodeContainerCache.clear();
	}

	/** {@inheritDoc} */
	@Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

	public void toggleShowOnlyIndividualAssociations() {
	    showOnlyIndividualAssociations = !showOnlyIndividualAssociations;
    }

}
